package controle;

import transporteUDP.Transmissao;
/*
 * Para calcular o sampleRTT (que est representado no cdigo como RTT), devemos
 * capturar o tempo no instante que o pacote comeca a ser enviado (chamada do
 *  de algum metodo "send") e tambm capturar o tempo no instante de chegada do ack
 *  correspondente quele arquivo. Isso deve ser feito da seguinte maneira:
 *  send(pacoteX);
 *  Long inicio = System.nanoTime();
 *  ....
 *  ....
 *  ack do pacoteX foi recebido;
 *  Long fim = System.nanoTime();
 *  
 *  o valor do rtt  a diferena entre o fim e o inicio!
 *  
 *  Vale lembrar que esses valores so em nanosegundos!
 *  para arrendonda-los e converte-los para milisegundos, deve-se usar o seguinte codigo:
 *  
 *  Math.round(((double)(data2 - dataAtual)/1000000)))
 *  
 *  SampleRTT: tempo medido a partir da transmisso do segmento at a recepo do ACK.
 *   ->> ignora retransmisses
 *  SampleRTT variar, deseja-se uma estimativa de RTT suavizada.
 *  ->> Faz-se uma mdia de vrias medidas recentes, no se usa somente a atual (SampleRTT)
 *  
 *  
 *  Timeout = EstimatedRTT + 4*DevRTT
 *  
 *  
 *  Setar valor inicial de devRTT e estimatedRTT para nao dar timeout no primeiro pacote!
 *  
 */

public class ControleCongestionamento {
	private double MSS;
	private double congWin;
	private double thresHold;
	private double sampleRTT;
	private double devRTT;
	private double estimatedRTT;
	private boolean perda;
	private boolean timeOut;
	private boolean reconhecido;
	private int estado;
	private int lastAck = -1;
	private int acksRepetidos = 1;
	private Transmissao transmissao;

	public ControleCongestionamento(double mss, Transmissao transmissao){
		//VALORES INICIAIS DO CONTROLE DE CONGESTIONAMENTO!
		this.MSS = mss;
		this.congWin = mss;
		this.thresHold = 999999999*mss;//valor alto, vai ser alterado aps a primeira perda!
		this.perda = false;
		this.timeOut = false;
		this.reconhecido = false;
		this.estado = 3; //Partida Lenta
		this.transmissao = transmissao;
	}

	public void setRTT(Double sampleRTT){
		this.sampleRTT = sampleRTT;
	}

	public void setTimeOut(boolean timeOut){
		this.timeOut = timeOut;
	}

	public double getCongWin(){
		return this.congWin;
	}

	public void receberAck(int numSeq){
		if(numSeq == this.lastAck){
			this.acksRepetidos++;
		}else{
			this.lastAck = numSeq;
			this.acksRepetidos = 1;
			this.reconhecido = true;
		}
		if(this.acksRepetidos == 3){
			this.perda = true;
			this.transmissao.ocorreuPerda(true);
		}
	}

	private void calcularDevRTT(){
		this.estimatedRTT = 0.875*this.estimatedRTT + 0.125*sampleRTT;
		this.devRTT = (0.75)*devRTT + (0.25)*Math.abs(sampleRTT - this.estimatedRTT);
	}

	public void run(){
		boolean sair = false;;
		while(true){
			if(this.timeOut | this.perda | this.reconhecido){
				this.calcularDevRTT();
				if(timeOut){//REAO A UM TIMEOUT
					this.thresHold = this.congWin/2;
					this.congWin = this.MSS;
					this.estado = 3;
					sair = true;
				}else if(perda){//REAO A UMA PERDA DE PACOTE (3 ACK'S IGUAIS)
					this.estado = 2;
				}
				//AUMENTO ADITIVO
				if(this.estado == 1){
					this.congWin = this.congWin + (this.MSS*this.MSS)/this.congWin;
					//comando acima resulta em aumento de 1MMS a cada RTT
					this.thresHold = this.congWin;
				}
				//DIMINUICAO MULTIPLICATIVA
				else if(this.estado == 2){
					if(this.congWin >= 2*this.MSS){
						this.congWin = this.congWin/2;
					}else this.congWin = this.MSS;
					this.thresHold = this.congWin;
					this.estado = 1;
				}
				//PARTIDA LENTA
				else if(this.estado == 3 && !sair){
					this.congWin = 2*this.congWin;
					if(this.congWin >= this.thresHold) this.estado = 1;
				}
			}
			this.perda = false;
			this.timeOut = false;
			this.reconhecido = false;
			sair = false;
		}
	}

	public boolean ocorreuPerda() {
		return this.perda;
	}
}