package transporte;

import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Vector;
import transporteTCP.ClienteTCP;
import transporteTCP.ServidorTCP;
import transporteUDP.ClienteUDP;
import transporteUDP.ServidorUDP;
import aplicacao.AplicacaoGUI;
import aplicacao.Usuario;

public class Cliente {
	private AplicacaoGUI interfaceGUI;
	private ClienteTCP clienteTCP;
	private ServidorTCP servidorTCP;
	private ClienteUDP clienteUDP;
	private ServidorUDP servidorUDP;
	private ClienteLogin servidorLogin;
	private String login;
	private boolean conectado;
	Usuario usuarioNovo;
	
	public Cliente(AplicacaoGUI aplicacao) throws IOException{
		this.interfaceGUI = aplicacao;
	}

	synchronized public boolean getConectado() {
		return this.conectado;
	}
	
	synchronized public void receberMsg(String readLine){
		this.interfaceGUI.msgRetorno(readLine);
	}
	
		
	synchronized public void receberMsgAck(Vector<String> msg){
		this.clienteUDP.receberAck(msg);
	}
	
	synchronized public void desconectarUsuario(Usuario novoUsuario) {
		if(!novoUsuario.getLogin().equalsIgnoreCase(this.login)){
			this.clienteTCP.desconectarUsuario(novoUsuario);
			this.servidorTCP.desconectarUsuario(novoUsuario);
			this.servidorUDP.desconectarUsuario(novoUsuario);
			this.clienteUDP.desconectarUsuario(novoUsuario);
		}
	}
	
	synchronized public void conectarUsuario(Usuario novoUsuario) throws UnknownHostException, IOException {
		if(!novoUsuario.getLogin().equalsIgnoreCase(this.login)){
			this.clienteTCP.conectarUsuario(novoUsuario);
			this.clienteUDP.conectarUsuario(novoUsuario);
		}
	}
	
	public void enviarMsg(String texto) throws IOException{
		this.clienteTCP.enviarMsg(texto);
	}

	public boolean conectar(String login, String senha) throws IOException {
		this.login = login;
		this.servidorLogin = new ClienteLogin(this,login,senha);
		if(this.conectado){
			this.servidorLogin.start();
			this.clienteTCP = new ClienteTCP(this);
			this.servidorTCP = new ServidorTCP(this);
			this.servidorTCP.start();
			this.clienteUDP = new ClienteUDP(this);
			this.servidorUDP = new ServidorUDP(this);
		}
		return this.conectado;
	}

	public Vector<Usuario> obterStatus() {
		return this.servidorLogin.getUsuarios();
	}

	public void desconectar() throws IOException {
		this.conectado = false;
		this.clienteTCP.desconectar();
		this.servidorTCP.desconectar();
	}

	public void setConectado(boolean status) {
		this.conectado = status;
		
	}
	
	public String getLogin(){
		return this.login;
	}

	public Usuario getNovoUsuario() {
		return this.usuarioNovo;
	}
	
	public void enviarArquivo(File arquivo) throws IOException{
		this.clienteUDP.enviarArquivo(arquivo);
	}

	public void nomeArquivo(String string, InetAddress ip) throws IOException {
		this.servidorUDP.nomeArquivo(string, ip);
	}
	
	
}
