package transporteUDP;
import java.io.File;
import java.io.IOException;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Vector;

import aplicacao.Usuario;

import transporte.Cliente;
import utilitario.Constantes;
import utilitario.Pacote;
import utilitario.Segmentador;

public class ClienteUDP {
	Vector<Transmissao> transmissao;
	Segmentador segmentador;
	ConexaoClienteUDP conexaoClienteUDP;
	Cliente cliente;
	
	public ClienteUDP(Cliente cliente) throws UnknownHostException, SocketException{
		transmissao = new Vector<Transmissao>();
		segmentador = new Segmentador();
		conexaoClienteUDP = new ConexaoClienteUDP();
		this.cliente = cliente;
	}
	
	public void receberAck(Vector<String> ack){
		String dono = ack.get(0);
		int index = 0;
		boolean achou = false;
		while(index < this.transmissao.size() && !achou){
			if(dono.equalsIgnoreCase(transmissao.get(index).getDono())){
				this.transmissao.get(index).receberAck(ack);	
				achou = true;
			} else {
				index++;
			}
		}
	}
	
	
	public void enviarArquivo(File file) throws IOException{
		int numDivisoes = this.segmentador.numeroDivisaoPacote(file);
		this.cliente.enviarMsg("enviarArquivo\n");
		this.cliente.enviarMsg(file.getName()+"\n");
		for(int i = 0; i < numDivisoes; i++){
			try {
				this.enviarPacotes(this.buscaArquivo(file, i+1));
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	//depois, os arquivos vao ser buscados por partes.
	private Pacote[] buscaArquivo(File file, int parte) throws IOException{
		//chama o segmentador
		
		Pacote[] pacotes = this.segmentador.segmentarArquivo(file, Constantes.tamanhoDoCabecalho, Constantes.tamanhoMaximoDoPacote, parte);

		return pacotes;
	}
	
	//pode ser feito com thread
	private void enviarPacotes(Pacote[] pacotes) throws IOException{
		for(int j = 0; j < this.transmissao.size(); j++){
			this.transmissao.get(j).pacotesASeremEnviado(pacotes);
			this.transmissao.get(j).start();
		}
	}

	public void desconectarUsuario(Usuario novoUsuario) {
		this.conexaoClienteUDP.desconectarUsuario(novoUsuario);
		int index = 0;
		boolean achou = false;
		while(index < this.transmissao.size() && !achou){
			if(novoUsuario.getLogin().equalsIgnoreCase(transmissao.get(index).getDono())){
				//se der pra desconectar uma transmissao, desconecta
				this.transmissao.remove(index);
				achou = true;
			} else {
				index++;
			}
		}
		
	}

	public void conectarUsuario(Usuario novoUsuario) throws SocketException, UnknownHostException {
		this.conexaoClienteUDP.conectarUsuario(novoUsuario);
		this.transmissao.add(new Transmissao(novoUsuario.getLogin(),this));
	}

	synchronized public void enviar(byte[] bs) throws IOException {
		conexaoClienteUDP.enviar(bs);
	}

	
}
