package transporteUDP;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.util.Vector;
import confiabilidade.Elemento;
import confiabilidade.JanelaDestinatario;
import controle.ControleDeFluxoDestinatario;
import utilitario.Condicoes;
import utilitario.Constantes;
import utilitario.Pacote;
import utilitario.BufferRecepcao;

public class Recepcao extends Thread {
    BufferRecepcao bufferDeChegada;
	ControleDeFluxoDestinatario controleDeFluxoDestinatario;
	JanelaDestinatario janelaRecepcao;
	ServidorUDP servidorUDP;
	private int porta;
	private InetAddress ip;
	private boolean conectado;
	private FileOutputStream fileOutputStream;
	
	public Recepcao(ServidorUDP servidorUDP, InetAddress ip, int porta){
		this.bufferDeChegada = new BufferRecepcao();
		this.controleDeFluxoDestinatario = new ControleDeFluxoDestinatario(Constantes.tamanhoDoBuffer);
		this.janelaRecepcao = new JanelaDestinatario();
		this.servidorUDP = servidorUDP;
		this.ip = ip;
		this.porta = porta;
		this.conectado = true;
	}
	
	public void run(){
		while(this.servidorUDP.getConectado() && this.conectado){
			if(this.servidorUDP.getAdicionado()){
				if(this.servidorUDP.getIpCliente().equals(this.ip) && (this.servidorUDP.getPortaCliente() == this.porta)){
					byte[] pck = this.servidorUDP.getPacote();
					this.servidorUDP.setAdicionado(false);
					try {
						//if(Checksum.verificarCheckSum(pck)){
							Pacote pacote = this.bytesToPacote(pck);
							this.colocarNoBuffer(pck);
							this.janelaRecepcao.reconhecer(pacote.getNumSeq());
							this.gerarAck(pacote);
							//verificar essa condio!
							if(this.bufferDeChegada.getQuantidadePacotes() > (Constantes.tamanhoDoBuffer/Constantes.tamanhoMaximoDoPacote)/2){
								this.getPacotesEmOrdemDoBuffer();
							}
						//}
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
		}
	}

	

	private Pacote bytesToPacote(byte[] pacoteEmBytes) {
		byte flag = pacoteEmBytes[0];
	
		short rcvWindow1 =(short) ((pacoteEmBytes[1] & 0xff) << 8);
		short rcvWindow2 = (short) (pacoteEmBytes[2] & 0xff);
		
		//rcvWindow
		short  rcvWindow =  (short) (rcvWindow1 + rcvWindow2);

		int numSeq1 = ((pacoteEmBytes[3] & 0xff) << 26);
		int numSeq2 = ((pacoteEmBytes[4] & 0xff) << 16);
		int numSeq3 = ((pacoteEmBytes[5] & 0xff) << 8);
		int numSeq4 = ((pacoteEmBytes[6] & 0xff));
		
		//numSeq
		int numSeq = numSeq1 + numSeq2 + numSeq3 + numSeq4;
		
		//checkSum
		short checkSum1 = (short) ((pacoteEmBytes[7] & 0xff) << 8);
		short checkSum2 = (short) ((pacoteEmBytes[8] & 0xff) << 8);
		
		short checkSum = (short) (checkSum1 + checkSum2);
		
		byte[] dados = new byte[pacoteEmBytes.length - 9];
		int index = 0;
		for(int i = 9; i < pacoteEmBytes.length; i++) {
			dados[index++] = pacoteEmBytes[i];
		}

		Pacote pacote = new Pacote(flag, rcvWindow, numSeq, checkSum, dados);
		return pacote;
	}
	
	private void colocarNoBuffer(byte[] pacote){//alteracao
		this.bufferDeChegada.setDadosBuffer(pacote);
	}

	public void enviarAck(int numSeq, int rcvWindow) throws IOException {
		this.servidorUDP.enviarMsg("ACK\n");
		this.servidorUDP.enviarMsg(numSeq+"\n");
		this.servidorUDP.enviarMsg(rcvWindow+"\n");
	}
	
	/**
	 * Metodo que retorna o pacote do buffer com numero de sequencia dado
	 * e o retira do buffer.
	 * */

	private Pacote getPacoteDoBuffer(){
		byte[] temp;
		
		byte flag = this.bufferDeChegada.getDadosBuffer(1)[0];
		
		temp = this.bufferDeChegada.getDadosBuffer(4);
		int numSeq = (temp[0] << 24) | (temp[1] << 16) | (temp[2] << 8) | temp[3];
		
		temp = this.bufferDeChegada.getDadosBuffer(2);
		short checkSum = (short) ((temp[0] << 8) | temp[1]);
		
		temp = this.bufferDeChegada.getDadosBuffer(2);
		short tamanho = (short) ((temp[0] << 8) | temp[1]);
		
		byte[] dados = this.bufferDeChegada.getDadosBuffer(tamanho - Constantes.tamanhoDoCabecalho);
		
		return new Pacote(flag, tamanho, numSeq, checkSum, dados);
	}
		

	
	/**
	 * Mtodo que retira do buffer os pacotes em ordem.
	 * retorna para AplicacaoServidor
	 * */
	
	public void getPacotesEmOrdemDoBuffer() {//(alteracao2)

		Vector<Integer> numerosSequencia = this.janelaRecepcao.getNumerosSequenciaEmOrdem();

		Pacote[] pacotesEmOrdem = new Pacote[numerosSequencia.size()];

		Pacote pacote;

		for(int i = 0; i < numerosSequencia.size(); i ++){

			pacotesEmOrdem[i] = this.getPacoteDoBuffer();

			pacote = pacotesEmOrdem[i];

			this.controleDeFluxoDestinatario.setLastByteRead(pacote.getTamanho()+pacote.getNumSeq());

		}

		if(numerosSequencia.size() != 0){

			this.servidorUDP.receberArquivo(pacotesEmOrdem,this.fileOutputStream);

		}

	}



	/**
	 * Metodo que calcula o tamanho dos pacotes que so "aguardados", mas ainda no foram recebidos.
	 * */		
	
	private int calculaEspacosVazios(){
		Elemento[] temp = this.janelaRecepcao.getJanela();
		int espacos = 0;
		for(int j = 0; j < this.janelaRecepcao.getTamanho(); j++) {
			if(temp[j].getCondicao() == Condicoes.AguardadoNaoRecebdo){
				espacos = espacos + temp[j+1].getNumeroSequencia() - temp[j].getNumeroSequencia();
			}
		}		
		return espacos;
	}
	
	public void gerarAck(Pacote pck) throws IOException{
		this.controleDeFluxoDestinatario.setLastByteRcvd(pck.getNumSeq() + pck.getTamanho());
		this.controleDeFluxoDestinatario.atualizarRcvWindow(this.calculaEspacosVazios());
		int rcvWindow = this.controleDeFluxoDestinatario.getRcvWindow();
		this.enviarAck(this.janelaRecepcao.getNumSeqEsperado(),rcvWindow);
	}

	public InetAddress getIp() {
		return this.ip;
	}
	
	public int getPorta(){
		return this.porta;
	}

	public void setConectado(boolean status){
		this.conectado = status;
	}

	public void setStream(String nome) throws IOException {
		File arquivo = new File("H:\\"+nome);
		arquivo.createNewFile();
		this.fileOutputStream = new FileOutputStream(arquivo);
	}

}
