package transporteUDP;

import java.io.IOException;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Vector;


import confiabilidade.JanelaRemetente;
import controle.ControleCongestionamento;
import controle.ControleDeFluxoRemetente;
import utilitario.Constantes;
import utilitario.Pacote;
import utilitario.Checksum;
import utilitario.Temporizador;


public class Transmissao extends Thread{
	private String dono;
	Vector<Pacote> bufferDeSaida;
	ControleDeFluxoRemetente controleDeFluxoRemetente;
	ControleCongestionamento controleCongestionamento;
	JanelaRemetente janelaTransmissao;
	ClienteUDP clienteUDP;
	Temporizador temporizador;
	Pacote[] pacotes;
	int ultimoAck;
	int ackRepetido;
	private boolean janelaAtualizada;
	
	public Transmissao(String dono, ClienteUDP clienteUDP) throws UnknownHostException, SocketException{
		this.controleDeFluxoRemetente = new ControleDeFluxoRemetente(25000);
		this.controleCongestionamento = new ControleCongestionamento(Constantes.MSS,this);
		this.bufferDeSaida = new Vector<Pacote>();
		this.janelaTransmissao = new JanelaRemetente();
		this.dono = dono;
		this.clienteUDP = clienteUDP;
		this.temporizador = new Temporizador(50);
	}
	
	public ControleCongestionamento getControleCongestionamento() {
		return controleCongestionamento;
	}

	public void setRcvWindow(int rcvWindow){
		this.controleDeFluxoRemetente.setRcvWindow(rcvWindow);
	}
	 // Mtodo chamado pela aplicacao cliente.
	
	
	public JanelaRemetente getJanelaTransmissao() {
		return janelaTransmissao;
	}

	public void setJanelatTransmissao(JanelaRemetente janelaTransmissao) {
		this.janelaTransmissao = janelaTransmissao;
	}

	/**
	 *  chamado pela aplicacao sempre que houver o desejo de envio de um pacote.
	 * Deve usar mtodos da Classe JanelaConfiabilidade para saber se  permitido
	 * colocar os pacotes da aplicao no buffer..
	 * */
	
	void colocarNoBuffer(Pacote pacote) {
		this.bufferDeSaida.add(pacote);
	}
	
	public ControleDeFluxoRemetente getControleDeFluxoRemetente() {
		return controleDeFluxoRemetente;
	}

	public void setControleDeFluxoRemetente(
			ControleDeFluxoRemetente controledeFluxoRemetente) {
		this.controleDeFluxoRemetente = controledeFluxoRemetente;
	}
	
	private boolean naoCongestionouNemTransbordouBuffer(){
		return (this.controleDeFluxoRemetente.getLastByteSent() - this.controleDeFluxoRemetente.getLastByteacked() 
				<= min(this.controleDeFluxoRemetente.getRcvWindow(), this.controleCongestionamento.getCongWin()));
	}

	private double min(int rcvWindow, double congWin) {
		double retorno;
		if(rcvWindow < congWin){
			retorno = rcvWindow;
		}else{
			retorno = congWin;
		}
		return retorno;
	}

	public void pacotesASeremEnviado(Pacote[] pacotes){
		this.pacotes = pacotes;
	}

	public void run(){
		boolean transmitindo = true;
		while(this.pacotes != null && transmitindo){
			//ESTADO DE ENVIO
			int proximoNumSeq = 0;
			int index = proximoNumSeq/Constantes.tamanhoMaximoDoPacote;
			while(proximoNumSeq != -1 && index < this.pacotes.length
					&& this.janelaTransmissao.estaNaJanela(this.pacotes[index].getNumSeq()) 
					&& this.naoCongestionouNemTransbordouBuffer()){
				
				if (this.controleCongestionamento.ocorreuPerda() || this.temporizador.getTimeOut()){
					proximoNumSeq = this.janelaTransmissao.getBase();
					this.controleCongestionamento.setTimeOut(true);
					this.temporizador.setTimeOut(false);
				}
				int lastByteSentAnterior = this.controleDeFluxoRemetente.getLastByteSent();
				this.controleDeFluxoRemetente.setLastByteSent(this.pacotes[index].getTamanho() + this.pacotes[index].getNumSeq());
				
				if(this.controleDeFluxoRemetente.getRcvWindow() > 0){
					
					this.janelaTransmissao.enviar(this.pacotes[index].getNumSeq());
					try {
						this.clienteUDP.enviar(this.pacoteToBytes(this.pacotes[index]));
						if(this.janelaAtualizada){
							this.temporizador.start();
							this.janelaAtualizada = false;
						}
						//this.colocarNoBuffer(this.pacotes[atualNaJanela]);
						proximoNumSeq = this.janelaTransmissao.getNextSeqNum();
					} catch (IOException e) {
						e.printStackTrace();
					}
					
				} else {
					this.controleDeFluxoRemetente.setLastByteSent(lastByteSentAnterior);
					//mandar pacote c 1 byte de dados
				}
			}
			//ESTADO DE ESPERA
			if(janelaAtualizada){
				proximoNumSeq = this.janelaTransmissao.getBase();
				this.janelaAtualizada = false;
			}
			index = proximoNumSeq/Constantes.tamanhoMaximoDoPacote;
		}
		
	}
	
	//metodos abaixo ainda n terminados
	/**
	 * Verifica se tem espaco no Buffer de recepcao e envia pacotes armazenados no buffer de saida.
	 * @throws IOException 
	 * */
	public boolean enviarPacote(Pacote pacote) throws IOException{//sempre chamado
		boolean retorno = false;
		//eu guardo o anterior, pq se nao enviar, controle de fluxo deve manter o valor anterior
		int lastByteSentAnterior = this.controleDeFluxoRemetente.getLastByteSent();
		this.controleDeFluxoRemetente.setLastByteSent(pacote.getTamanho() + pacote.getNumSeq());
		
		if(this.janelaTransmissao.estaNaJanela(pacote.getNumSeq())
				&& this.naoCongestionouNemTransbordouBuffer()){
			if(this.controleDeFluxoRemetente.getRcvWindow() > 0){
				this.janelaTransmissao.enviar(pacote.getNumSeq());
				pacote.setCheckSum(Checksum.calcCheckSum(pacote.getFlag(), pacote.getTamanho(), pacote.getNumSeq(), pacote.getParte()));
				this.clienteUDP.enviar(this.pacoteToBytes(pacote));
				
				if(this.janelaAtualizada){
					this.temporizador.start();
					this.janelaAtualizada = false;
				}
				this.colocarNoBuffer(pacote);
				retorno = true;
			} else {
				this.controleDeFluxoRemetente.setLastByteSent(lastByteSentAnterior);
				//mandar pacote c 1 byte de dados
			}
		}
		return retorno;
	}
	
	public byte[] pacoteToBytes(Pacote pacote) {
		byte[] retorno = new byte[pacote.getTamanho()];
		
		retorno[0] = pacote.getFlag();
		
		retorno[1] = (byte) (pacote.getTamanho() >>> 8);
		retorno[2] = (byte) (pacote.getTamanho() & 0xFF);
		
		retorno[3] = (byte) (pacote.getNumSeq() >>> 24);
		retorno[4] = (byte) ((pacote.getNumSeq() >>> 16) & 0xff);
		retorno[5] = (byte) ((pacote.getNumSeq() >>> 8) & 0xff);
		retorno[6] = (byte) (pacote.getNumSeq() & 0xff);
	
		retorno[7] = (byte) (pacote.getCheckSum() >>> 8);
		retorno[8] = (byte) (pacote.getCheckSum() & 0xff);
		
		int tamDados = pacote.getParte().length;
		
		for(int i = 0; i < tamDados; i++) retorno[i + 9] = pacote.getParte()[i];

		
		return retorno;
	}


	/**
	 *  chamado quando o remetente recebe um pacote(Ack).
	 * */
	void  receberAck(Pacote pkg){
		this.controleDeFluxoRemetente.setRcvWindow(pkg.getTamanho());
		this.controleDeFluxoRemetente.setLastByteacked(pkg.getNumSeq());
		
	}

	public void receberAck(Vector<String> ack) {
		int numSeq = Integer.parseInt(ack.get(1));
		this.controleCongestionamento.receberAck(numSeq);
		if(this.janelaTransmissao.getBase() < numSeq){
			int rcvWindow = Integer.parseInt(ack.get(2));
			this.controleDeFluxoRemetente.setLastByteacked(numSeq);
			this.controleDeFluxoRemetente.setRcvWindow(rcvWindow);
			this.temporizador.setAck(true);
			this.janelaTransmissao.reconhecer(numSeq);
			this.janelaAtualizada = true;
		}
	}

	public String getDono() {
		return this.dono;
	}

	public void ocorreuPerda(boolean perda) {
		
	}

}
